#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::unit::AmazonWidgetUtil
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for AmazonWidgetUtil
#========================================================================================
package DE_EPAGES::AmazonPayments::t::unit::AmazonWidgetUtil;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use DE_EPAGES::AmazonPayments::API::AmazonWidgetUtil;

use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_getAmazonWidgetUrl();
    $self->test_getAmazonWidgetUrl_InTestMode();
    $self->test_getAmazonButtonUrl();
    $self->test_getAmazonButtonUrl_InTestMode();
    $self->test_getActiveAmazonRegion();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getAmazonWidgetUrl {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethod = $self->memory()->get('PaymentMethod');
    my $Region = 'DE';
    my $SUT = $self->memory()->get('SUT');

    my $WidgetUrl = $SUT->getAmazonWidgetUrl($PaymentMethod, $Region);

    is($WidgetUrl, 'https://static-eu.payments-amazon.com/OffAmazonPayments/de/js/Widgets.js?sellerId=someId',"$SubTestName - Test if WidgetUrl is as expected");
}
#----------------------------------------------------------------------------------------
sub test_getAmazonWidgetUrl_InTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethod = $self->memory()->get('PaymentMethodInTestMode');
    my $Region = 'DE';
    my $SUT = $self->memory()->get('SUT');

    my $WidgetUrl = $SUT->getAmazonWidgetUrl($PaymentMethod, $Region);

    is($WidgetUrl, 'https://static-eu.payments-amazon.com/OffAmazonPayments/de/sandbox/js/Widgets.js?sellerId=someId',"$SubTestName - Test if WidgetUrl is as expected in testmode");
}
#----------------------------------------------------------------------------------------
sub test_getAmazonButtonUrl {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethod = $self->memory()->get('PaymentMethod');
    my $Region = 'DE';
    my $SUT = $self->memory()->get('SUT');

    my $WidgetUrl = $SUT->getAmazonButtonUrl($PaymentMethod, $Region);

    is($WidgetUrl, 'https://payments.amazon.de/gp/widgets/button?sellerId=someId&size=large&color=tan',"$SubTestName - Test if WidgetUrl is as expected");
}
#----------------------------------------------------------------------------------------
sub test_getAmazonButtonUrl_InTestMode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethod = $self->memory()->get('PaymentMethodInTestMode');
    my $Region = 'DE';
    my $SUT = $self->memory()->get('SUT');

    my $WidgetUrl = $SUT->getAmazonButtonUrl($PaymentMethod, $Region);

    is($WidgetUrl, 'https://payments-sandbox.amazon.de/gp/widgets/button?sellerId=someId&size=large&color=tan',"$SubTestName - Test if WidgetUrl is as expected");
}

sub test_getActiveAmazonRegion {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PaymentMethod = $self->memory()->get('PaymentMethod');
    my $Basket = $self->memory()->get('Basket');
    my $SUT = $self->memory()->get('SUT');

    my $Region = $SUT->getActiveAmazonRegion($PaymentMethod, $Basket);

    is($Region, 'someRegion',"$SubTestName - Test if Region is as expected");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $SUT = $self->_createSUT();
    my $PaymentMethod = $self->_createPaymentMethod({'IsInTestMode'=>undef});
    my $PaymentMethodInTestMode = $self->_createPaymentMethod({'IsInTestMode'=>'1'});
    my $LineItemContainer = $self->_createLineItemContainer();
    my $Basket = $self->_createBasket($LineItemContainer);

    $self->memory()->add('SUT', $SUT);
    $self->memory()->add('Basket', $Basket);
    $self->memory()->add('PaymentMethod', $PaymentMethod);
    $self->memory()->add('PaymentMethodInTestMode', $PaymentMethodInTestMode);
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    return DE_EPAGES::AmazonPayments::API::AmazonWidgetUtil->new();
}

sub _createBasket {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $hAttributes = {
        'LineItemContainer' => $LineItemContainer
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'LocaleID' => 'de_DE',
        'CurrencyID' => 'EUR'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createPaymentMethod {
    my $self = shift;
    my ($hAdditionalAttributes) = @_;

    my $hAttributes = {
        'ButtonColor' => 'tan',
        'MerchantID' => 'someId',
        %{$hAdditionalAttributes}
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments',
        $hAttributes
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('regionForSF', 'someRegion', [ {'string'=>'de_DE'}, {'string'=>'EUR'}]);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();