#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::unit::PaymentMethodAmazonPayments
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments
#========================================================================================
package DE_EPAGES::AmazonPayments::t::unit::PaymentMethodAmazonPayments;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_callbackUrl();
    $self->test_getAlternativeCheckoutButtonInfo();
    $self->test_getAlternativeCheckoutButtonInfo_CanNotAddToBasket();
    $self->test_getAlternativeCheckoutButtonInfo_OrderAmountIsNotReached();

    done_testing();
}

sub test_callbackUrl {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WebServerScriptNamePart = 'MyShop';
    my $CanAddToBasket = 1;
    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($WebServerScriptNamePart, $CanAddToBasket);

    my $Url = $PaymentMethod->callbackUrl();

    is($Url, 'https://epagesj/rs/shops/MyShop/payments/amazon/callback/', "$SubTestName - test if callback url is returned");
}

sub test_getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WebServerScriptNamePart = 'MyShop';
    my $CanAddToBasket = 1;
    my $IsMinOrderAmountReached = 1;

    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($WebServerScriptNamePart, $CanAddToBasket);
    my $Basket = $self->_createBasket($IsMinOrderAmountReached);

    my $hInfo = $PaymentMethod->getAlternativeCheckoutButtonInfo($Basket);
    my $hExpectedInfo = {
        'name' => 'amazonpayments',
        'image' => 'http://button.url',
        'dataMerchantId' => 'someMerchantId',
        'widgetUrl' => 'http://widget.url'
    };

    is_deeply($hInfo, $hExpectedInfo, "$SubTestName - Test if returned hash is as expected");
}

sub test_getAlternativeCheckoutButtonInfo_CanNotAddToBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WebServerScriptNamePart = 'MyShop';
    my $CanAddToBasket = 0;
    my $IsMinOrderAmountReached = 1;

    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($WebServerScriptNamePart, $CanAddToBasket);
    my $Basket = $self->_createBasket($IsMinOrderAmountReached);

    my $hInfo = $PaymentMethod->getAlternativeCheckoutButtonInfo($Basket);

    is($hInfo, undef, "$SubTestName - Test if undef is returned if can not addtobasket");
}

sub test_getAlternativeCheckoutButtonInfo_OrderAmountIsNotReached {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $WebServerScriptNamePart = 'MyShop';
    my $CanAddToBasket = 1;
    my $IsMinOrderAmountReached = 0;

    my $PaymentMethod = $self->_createPaymentMethodAmazonPayments($WebServerScriptNamePart, $CanAddToBasket);
    my $Basket = $self->_createBasket($IsMinOrderAmountReached);

    my $hInfo = $PaymentMethod->getAlternativeCheckoutButtonInfo($Basket);

    is($hInfo, undef, "$SubTestName - Test if undef is returned if can not addtobasket");
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodAmazonPayments {
    my $self = shift;
    my ($WebServerScriptNamePart, $CanAddToBasket) = @_;

    my $hParams = {
        'MerchantID' => 'someMerchantId'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
       'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments',
       $hParams
    );
    $Mockify->addMockWithReturnValue('getSite', $self->_createSite($WebServerScriptNamePart));
    $Mockify->addMockWithReturnValue('_createShopUriBuilder', $self->_createShopUriBuilder($WebServerScriptNamePart));
    $Mockify->addMockWithReturnValueAndParameterCheck('canAddToBasket', $CanAddToBasket, [{
        'hashref' => {
            'LocaleID' => 'someLocale',
            'CurrencyID' => 'someCurrency'
         }
    }]);
    $Mockify->addMockWithReturnValue('_createAmazonWidgetUtil', $self->_createAmazonWidgetUtil());
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopUriBuilder {
    my $self = shift;
    my ($WebServerScriptNamePart) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Shop::API::ShopUriBuilder',
        [$self->_createSite(), $self->_createSystem()]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('buildUri', 'https://epagesj/rs/shops/MyShop/payments/amazon/callback/', [
        {'string' => 'payments/amazon/callback/'}
    ]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;
    my ($WebServerScriptNamePart) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop',
        'PathString' => '/test/path',
        'RESTAPIURL' => 'https://publicEpages/test/path',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'http://localhost:8088/rs/ '
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
sub _createAmazonWidgetUtil {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AmazonPayments::API::AmazonWidgetUtil',
        $aParameterList
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('getActiveAmazonRegion', 'someRegion', [
        {'object' => 'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments'},
        {'object' => 'DE_EPAGES::Order::API::Object::Basket'}
    ]);
    $Mockify->addMockWithReturnValueAndParameterCheck('getAmazonWidgetUrl', 'http://widget.url', [
        {'object' => 'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments'},
        {'string' => 'someRegion'}
    ]);
    $Mockify->addMockWithReturnValueAndParameterCheck('getAmazonButtonUrl', 'http://button.url', [
        {'object' => 'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments'},
        {'string' => 'someRegion'}
    ]);

    return $Mockify->getMockObject();
}

sub _createBasket {
    my $self = shift;
    my ($IsMinOrderAmountReached) = @_;

    my $hAttributes = {
        'IsMinOrderAmountReached' => $IsMinOrderAmountReached,
        'LineItemContainer' => $self->_createLineItemContainer()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'LocaleID' => 'someLocale',
        'CurrencyID' => 'someCurrency'
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();