#!perl -w
use strict;
use Config;
use Test::More tests => 4;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::Error  qw( GetError ExistsError );
use DE_EPAGES::Core::API::File   qw( WriteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Net qw(  GetFastestEpagesMirror);

sub Main {

    # GetFastestEpagesMirror

    my $Version = '6.15.3';
    my $PatchFile = "patch/$Version/epages-$Version-patch-data.tar.gz";
    my $Url = GetFastestEpagesMirror($PatchFile);
    # value of $Url depends on connection speed to different epages mirrors
    ok($Url, "GetFastestEpagesMirror($PatchFile) -> $Url");

    $Version = '6.0.9';
    $PatchFile = "patch/$Version/epages-$Version-patch-data.tar.gz";
    $Url = eval { GetFastestEpagesMirror($PatchFile) };
    my $Error = ExistsError() ? GetError() : undef;
    ok($Error->code eq 'GET_FASTEST_MIRROR_FAILED',
       "GetFastestEpagesMirror($PatchFile) -> ". $Error->code);

    my $TempDir = tempdir( CLEANUP => 1 );
    local $ENV{EPAGES_PATCH_DATA_REPO} = $TempDir;
    my $Expected = "$ENV{EPAGES_PATCH_DATA_REPO}/$PatchFile";
    WriteFile($Expected, \1, { create_directory=>1});

    $Url = GetFastestEpagesMirror($PatchFile);
    ok($Url eq $Expected, "EPAGES_PATCH_DATA_REPO=$ENV{EPAGES_PATCH_DATA_REPO} GetFastestEpagesMirror($PatchFile) -> $Url");

    $PatchFile = 'rpmify/rpmify-epages.sh';
    local $ENV{EPAGES_PATCH_DATA_REPO} = 'http://epages-software.de/RPMS';
    $Expected = "$ENV{EPAGES_PATCH_DATA_REPO}/$PatchFile";
    $Url = GetFastestEpagesMirror($PatchFile);
    ok($Url eq $Expected, "EPAGES_PATCH_DATA_REPO=$ENV{EPAGES_PATCH_DATA_REPO} GetFastestEpagesMirror($PatchFile) -> $Url");
}

RunScript(Sub=>\&Main, Debug=>1);
