#!/bin/bash -x

export EPAGES_BACKUP_DIR=
export EPAGES_BACKUP_DB=
export EPAGES_INSTALL_VERSION=$2

# return values

NO_CURL=101
RPMIFY_FAILED=102
GET_RPMIFY_FAILED=103

APT_PATCH_FAILED=110
YUM_PATCH_FAILED=105
ZYPPER_PATCH_FAILED=104
APT_UPGRADE_FAILED=109
YUM_UPGRADE_FAILED=114

GET_REPOSITORY_FAILED=111
SET_REPOSITORY_FAILED=108
UPDATE_REPOSITORY_FAILED=112

HOTFIX_CONSOLE_FAILED=106
INSTALL_HOTFIX_FAILED=107
INSTALL_HOTFIX_BASE_FAILED=113

# export variables

export EPAGES_BACKUP_DIR=
export EPAGES_BACKUP_DB=
export EPAGES_INSTALL_VERSION=$2

type curl >/dev/null || exit $NO_CURL

if [[ -s /etc/yum/vars/EPAGES_REPO ]] ; then
  # if file /etc/yum/vars/EPAGES_REPO has content, use this
  EPAGES_REPOSITORY=$(cat /etc/yum/vars/EPAGES_REPO | head -1)
  EPAGES_REPOSITORY=$(echo $EPAGES_REPOSITORY | sed 's, .*,,')
fi

# patch version is older than 6.17.5 -> old method

if [[ $(printf "%d%03d%03d" ${EPAGES_INSTALL_VERSION//./ }) -lt 6017005 ]] ; then

  EPAGES_REPOSITORY=${EPAGES_REPOSITORY:-ftp://epages-software.de/RPMS}

  curl -Ls $EPAGES_REPOSITORY/rpmify/rpmify-epages.sh | sh || exit $RPMIFY_FAILED
  [[ ${PIPESTATUS[0]} = 0 ]] || exit $GET_RPMIFY_FAILED

  if [[ -f /etc/SuSE-release ]] ; then
    zypper modifyrepo --enable epages-update-$2
    zypper update -y -r epages-update-$2 epages-j
    zypper update -y -r epages-update-$2 epages-hotfix-base
    zypper update -y -r epages-update-$2 epages-patch && RET=0 || RET=$ZYPPER_PATCH_FAILED
    zypper modifyrepo --disable epages-update-$2
    exit $RET
  else
    yum -y update epages-j --disablerepo epages-update --enablerepo epages-update-$2
    yum -y update epages-hotfix-base --disablerepo epages-update --enablerepo epages-update-$2
    yum -y update epages-patch --disablerepo epages-update --enablerepo epages-update-$2 \
    || exit $YUM_PATCH_FAILED

    # install latest hotfixes
    DN=$(cd $(dirname $0) && pwd)
    InstallHotfixBase=${DN%/*}/customize/ZZ-InstallHotfixBase.sh
    if rpm --quiet -q epages-hotfix-base ; then
      /etc/cron.daily/epages-hotfix -console || exit $HOTFIX_CONSOLE_FAILED
    elif [[ -f $InstallHotfixBase ]] ; then
      chmod +x $InstallHotfixBase
      $InstallHotfixBase || exit $INSTALL_HOTFIX_FAILED
    fi
  fi

  exit 0
fi

# patch version is 6.17.5 or newer -> new method

EPAGES_REPOSITORY=${EPAGES_REPOSITORY:-ftp://epages-software.de/repo/epages}

if [[ -f /etc/debian_versions ]] ; then

  curl -LsO $EPAGES_REPOSITORY/apt/epages-release_all.deb || exit $GET_REPOSITORY_FAILED
  dpkg -i epages-release_all.deb || exit $UPDATE_REPOSITORY_FAILED
  /var/epages/SetRepository.sh -b $2 -r $EPAGES_REPOSITORY || exit $SET_REPOSITORY_FAILED

  apt-get --only-upgrade install 'epages-[a-oq-z]+' || exit $APT_UPGRADE_FAILED
  apt-get install epages-patch || exit $APT_PATCH_FAILED

  rm -f /etc/yum.repos.d/epages-hotfix.repo

elif [[ -f /etc/redhat-release ]] ; then

  rpm -hUv $EPAGES_REPOSITORY/yum/epages-release.noarch.rpm 2>&1 \
  | tee /tmp/runPatch-update-epages-release.log
  [[ ${PIPESTATUS[0]} = 0 ]] || grep -q 'already installed' \
  /tmp/runPatch-update-epages-release.log || exit $UPDATE_REPOSITORY_FAILED

  /var/epages/SetRepository.sh -b $2 -r $EPAGES_REPOSITORY || exit $SET_REPOSITORY_FAILED

  yum -y update 'epages-[a-oq-z]*' --disablerepo epages-setup --enablerepo epages-update \
  || exit $YUM_UPGRADE_FAILED
  yum -y update epages-patch --disablerepo epages-setup --enablerepo epages-update || {
    PATCH_RESULT="$EPAGES"/Shared/Log/epages-patch.result
    [[ -f $PATCH_RESULT && $(cat $PATCH_RESULT) = 0 ]] || exit $YUM_PATCH_FAILED
  }
fi

exit 0
