package DE_EPAGES::Order::t::CartCheckoutButtonResource_Get;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Order::API::Resource::CartCheckoutButton;
use URI::QueryParam;
use URI::Escape;
use URI;
use Test::More;
use Test::Exception;
use Data::Dumper;

use strict;

sub importXMLPath {
    shift;
    return 'cartCheckoutButtonTest.xml';
}

sub shopAlias {
    shift;
    return 'TestShop_CartCheckoutButton';
}

## no critic qw( ProhibitMagicNumbers )

sub test {
    my $self = shift;

    $self->test_Payments();

    #place her later on the delete and put test
    done_testing();
}

sub test_Payments {
    my $self = shift;

    my $UserAlias = 'cartTester01';
    my $BasketAlias = 'cartTest01';
    my $Cart = $self->shop->folder('AnonymousUsers')->child($UserAlias)->folder('Baskets')->child($BasketAlias);
    my $Resource = DE_EPAGES::Order::API::Resource::CartCheckoutButton->new();

    my $Request = HTTP::Request->new(GET => 'http://www.example.com/api/carts/' . $Cart->guid . '/checkoutbutton');
    ok($Resource->match($Request), 'url1 matches');

    my $hData = $Resource->get( $Cart );
    ok $hData, "get cart";

    ok (defined $hData->{'items'}->[1]->{'image'}, 'check if generated value is defined');
    $hData->{'items'}->[1]->{'image'} = 'https://someboilerplate';

    ok (defined $hData->{'items'}->[1]->{'paymentMethodId'}, 'check if generated value is defined');
    $hData->{'items'}->[1]->{'paymentMethodId'} = 'generateId';

    my $hExpected = $self->_expectedPayments();
    is_deeply( $hData, $hExpected, 'cart result deeply checked') or diag Dumper($hData);
}

sub _expectedPayments {
    return { 'items' =>
        [
            { 'dataMerchantId' => 'MyMerchantId',
              'widgetUrl' => 'https://static-eu.payments-amazon.com/OffAmazonPayments/de/sandbox/js/Widgets.js?sellerId=MyMerchantId',
              'name' => 'amazonpayments',
              'image' => 'https://payments-sandbox.amazon.de/gp/widgets/button?sellerId=MyMerchantId&size=large&color=tan'
            }, 
            { 'paymentMethodId' => 'generateId',
              'name' => 'paypalexpress',
              'image' => 'https://someboilerplate'
            }
        ]};
}

__PACKAGE__->runScript;
