#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::IconUrlLocalizer
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for ButtonUrlLocalizer
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::IconUrlLocalizer;
use base DE_EPAGES::Test::API::UnitTest::Test;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use strict;

sub test {
    my $self = shift;

    $self->test_getPayPalExpressIconUrl_german_silver();
    $self->test_getPayPalExpressIconUrl_german_gold();
    $self->test_getPayPalExpressIconUrl_notgerman();
    $self->test_getPayPalExpressIconUrl_notgerman_NoLocale();
    $self->test_getPayPalExpressIconUrl_notgerman_WrongLocale();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getPayPalExpressIconUrl_german_silver {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Language = 1;
    my $Locale = 'de_DE';
    my $IsPayPalExpressButtonSilver = 1;

    my $SUT = $self->_createSUT();
    my $Basket = $self->_createBasket($Language, $Locale);
    my $PaymentMethod = $self->_createPaymentMethod($IsPayPalExpressButtonSilver);

    my $LocalizedUrl = $SUT->getPayPalExpressIconUrl($Basket, $PaymentMethod);

    is($LocalizedUrl,'http://server.url/StoreRoot/SF/ico_expresscheckout_silver_de.gif',"$SubTestName - Test if URL has silver image");
}
#----------------------------------------------------------------------------------------
sub test_getPayPalExpressIconUrl_german_gold {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Language = 1;
    my $Locale = 'de_DE';
    my $IsPayPalExpressButtonSilver = 0;

    my $SUT = $self->_createSUT();
    my $Basket = $self->_createBasket($Language, $Locale);
    my $PaymentMethod = $self->_createPaymentMethod($IsPayPalExpressButtonSilver);

    my $LocalizedUrl = $SUT->getPayPalExpressIconUrl($Basket, $PaymentMethod);

    is($LocalizedUrl,'http://server.url/StoreRoot/SF/ico_expresscheckout_gold_de.gif',"$SubTestName - Test if URL has gold image");
}
#----------------------------------------------------------------------------------------
sub test_getPayPalExpressIconUrl_notgerman {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Language = 2;
    my $Locale = 'fr_FR';
    my $IsPayPalExpressButtonSilver = 0;

    my $SUT = $self->_createSUT();
    my $Basket = $self->_createBasket($Language, $Locale);
    my $PaymentMethod = $self->_createPaymentMethod($IsPayPalExpressButtonSilver);

    my $LocalizedUrl = $SUT->getPayPalExpressIconUrl($Basket, $PaymentMethod);

    is($LocalizedUrl,'https://url/fr_FR/localized',"$SubTestName - Test if URL was localized correctly");
}
#----------------------------------------------------------------------------------------
sub test_getPayPalExpressIconUrl_notgerman_NoLocale {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Language = 2;
    my $Locale = undef;
    my $IsPayPalExpressButtonSilver = 0;

    my $SUT = $self->_createSUT();
    my $Basket = $self->_createBasket($Language, $Locale);
    my $PaymentMethod = $self->_createPaymentMethod($IsPayPalExpressButtonSilver);

    my $LocalizedUrl = $SUT->getPayPalExpressIconUrl($Basket, $PaymentMethod);

    is($LocalizedUrl,'https://url/en_US/localized',"$SubTestName - Test if US is used as fallback");
}
#----------------------------------------------------------------------------------------
sub test_getPayPalExpressIconUrl_notgerman_WrongLocale {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Language = 2;
    my $Locale = 'dasIstSicherKeinValiderLocale';
    my $IsPayPalExpressButtonSilver = 0;

    my $SUT = $self->_createSUT();
    my $Basket = $self->_createBasket($Language, $Locale);
    my $PaymentMethod = $self->_createPaymentMethod($IsPayPalExpressButtonSilver);

    my $LocalizedUrl = $SUT->getPayPalExpressIconUrl($Basket, $PaymentMethod);

    is($LocalizedUrl,'https://url/en_US/localized',"$SubTestName - Test if US is used as fallback");
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalExpress::API::IconUrlLocalizer',
        $aParameterList
    );
    $Mockify->addMockWithReturnValue('_LoadRootObject', $self->_createSystem());

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($Language, $Locale) = @_;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer($Language, $Locale),
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($LanguageID, $Locale) = @_;

    my $hAttributes = {
        'LanguageID' => $LanguageID,
        'LocaleID' => $Locale
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'ProtocolAndServerSSL' => 'http://server.url',
        'StoreRoot' => '/StoreRoot',
        'PayPalExpress_CheckoutButtonURL' => 'https://url/[[locale]]/localized',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;
    my ($IsPayPalExpressButtonSilver) = @_;

    my $hAttributes = {
        'IsPayPalExpressButtonSilver' => $IsPayPalExpressButtonSilver
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();