#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::PaymentMethodPayPalExpress
# base         base DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  Unit test for PaymentMethodPayPalExpress
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::PaymentMethodPayPalExpress;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_setDefaultNameForAllLanguages_ValidDefaultName();
    $self->test_getAlternativeCheckoutButtonInfo();
    $self->test_getAlternativeCheckoutButtonInfo_IsHidden();
    $self->test_getAlternativeCheckoutButtonInfo_CanNotAddToBasket();
    $self->test_getAlternativeCheckoutButtonInfo_MinOrderAmountNotReached();

    done_testing();
}

# Hard Coded String PayPal
sub test_setDefaultNameForAllLanguages_ValidDefaultName {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_buildSUT();

    is($SUT->get('Name', 1), 'Any name for DE', "$SubTestName - Test if name DE is initialized properly.");
    is($SUT->get('Name', 2), 'Any name for EN', "$SubTestName - Test if name EN is initialized properly.");

    $SUT->_setDefaultNameForAllLanguages(); ## no critic (ProhibitPrivateSubroutineUse)

    is($SUT->get('Name', 1), 'PayPal', "$SubTestName - Test if name DE was set to default name when present.");
    is($SUT->get('Name', 2), 'PayPal', "$SubTestName - Test if name EN was set to default name when present.");
}

sub test_getAlternativeCheckoutButtonInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hBasketConfig = { 'IsMinOrderAmountReached' => 1};
    my $hSUTConfig = { 
        'HidePayPalExpress' => 0,
        'CanAddToBasket' => 1
    };

    my $SUT = $self->_buildSUT($hSUTConfig);
    my $Basket = $self->_createBasket($hBasketConfig);

    my $hInfo = $SUT->getAlternativeCheckoutButtonInfo($Basket);
    my $hExpectedInfo = {
        'name' => 'paypalexpress',
        'image' => 'http://icon.url',
        'paymentMethodId' => 'PaymentMethodId',
    };

    is_deeply($hInfo, $hExpectedInfo, "$SubTestName - Test if infohash was as expected.");
}

sub test_getAlternativeCheckoutButtonInfo_IsHidden {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hBasketConfig = { 'IsMinOrderAmountReached' => 1};
    my $hSUTConfig = { 
        'HidePayPalExpress' => 1,
        'CanAddToBasket' => 1
    };

    my $SUT = $self->_buildSUT($hSUTConfig);
    my $Basket = $self->_createBasket($hBasketConfig);

    my $hInfo = $SUT->getAlternativeCheckoutButtonInfo($Basket);

    is($hInfo, undef, "$SubTestName - Test if info is not returned.");
}

sub test_getAlternativeCheckoutButtonInfo_CanNotAddToBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hBasketConfig = { 'IsMinOrderAmountReached' => 1};
    my $hSUTConfig = { 
        'HidePayPalExpress' => 0,
        'CanAddToBasket' => 0
    };

    my $SUT = $self->_buildSUT($hSUTConfig);
    my $Basket = $self->_createBasket($hBasketConfig);

    my $hInfo = $SUT->getAlternativeCheckoutButtonInfo($Basket);

    is($hInfo, undef, "$SubTestName - Test if info is not returned.");
}

sub test_getAlternativeCheckoutButtonInfo_MinOrderAmountNotReached {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hBasketConfig = { 'IsMinOrderAmountReached' => 0};
    my $hSUTConfig = { 
        'HidePayPalExpress' => 0,
        'CanAddToBasket' => 1
    };

    my $SUT = $self->_buildSUT($hSUTConfig);
    my $Basket = $self->_createBasket($hBasketConfig);

    my $hInfo = $SUT->getAlternativeCheckoutButtonInfo($Basket);

    is($hInfo, undef, "$SubTestName - Test if info is not returned.");
}

sub _buildSUT {
    my $self = shift;
    my ($hConfig) = @_;

    my $PackageName = 'DE_EPAGES::PayPalExpress::API::Object::PaymentMethodPayPalExpress';
    my $hAttributes = {
        'Shop' => $self->_createShop(),
        'ID' => 'PaymentMethodId',
        'HidePayPalExpress' => $hConfig->{'HidePayPalExpress'}
    };
    my $DefaultLanguageId = 1;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId);
    $Mockify->addLanguageDependentAttribute('Name', {
        1 => 'Any name for DE',
        2 => 'Any name for EN',
    });
    $Mockify->addMockWithReturnValue('_createIconUrlLocalizer', $self->_createIconUrlLocalizer());
    $Mockify->addMockWithReturnValue('canAddToBasket', $hConfig->{'CanAddToBasket'});

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $PackageName = 'DE_EPAGES::Payment::API::Object::PaymentType';
    my $hAttributes = {
        'Languages' => [{'LanguageID' => 1}, {'LanguageID' => 2}],
    };
    my $DefaultLanguageId = 1;

    return DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId)->getMockObject();
}

sub _createBasket {
    my $self = shift;
    my ($hConfig) = @_;

    my $hAttributes = {
        'IsMinOrderAmountReached' => $hConfig->{'IsMinOrderAmountReached'},
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createIconUrlLocalizer {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalExpress::API::IconUrlLocalizer',
        $aParameterList
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('getPayPalExpressIconUrl', 'http://icon.url', [{'object' => 'DE_EPAGES::Order::API::Object::Basket'},'object']);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();