#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::ExperienceProfileId_Repository
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for ExperienceProfileId/Repository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::ExperienceProfileId_Repository;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Repository;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use Test::Exception;
use strict;
use DE_EPAGES::Test::Mock::EpagesObject;

sub test {
    my $self = shift;
    $self->test_SuccessfulCollection();
    $self->test_Error();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_SuccessfulCollection {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createExperienceProfileId_Repository();
    my $Shop = $self->_createShop();
    my $PaymentMethodPaypalPlus = $self->_createPaymentMethodPaypalPlus($Shop);

    lives_ok(sub {$Repository->buildRequest($PaymentMethodPaypalPlus)}, "$SubTestName - test if the Data could be collected");
    my $hResponse = {'id' => 'someid'};
    lives_ok(sub {$Repository->buildResponse($hResponse)}, "$SubTestName - test if the Data could be collected");
    lives_ok(sub {$Repository->writeResponse($PaymentMethodPaypalPlus);}, "$SubTestName - test if the Response could be written");

    my $RequestDTO = $Repository->getRequestDTO();
    my $ResponseDTO = $Repository->getResponseDTO();

    is($RequestDTO->localCode(), 'DE', "$SubTestName - tests get of localCode");
    like($RequestDTO->profileName(), qr/^[0-9A-Z\-]*$/, "$SubTestName - tests if profilename is GUID");
    is($RequestDTO->brandName(), 'MyShopName', "$SubTestName - tests get of brandname");
    is($RequestDTO->allowNote(), 'true', "$SubTestName - tests get of isNoteAllowed");
    is($RequestDTO->shippingAddressVisibility(), 2, "$SubTestName - tests get of showShippingAddress");
    is($RequestDTO->addressOverride(), 1, "$SubTestName - tests get of addressOverride");
    is($ResponseDTO->experienceProfileID(), 'someid', "$SubTestName - tests get of experienceProfileID");
    is($PaymentMethodPaypalPlus->get('ExperienceProfileId'), 'someid',  "$SubTestName - tests get of experienceProfileID in PaymentMethod");

    my $CredentialDTO = $Repository->getCredentialDTO();
    is($CredentialDTO->providerClientId(), 'someId', "$SubTestName - tests get of clientId");
    is($CredentialDTO->providerClientSecret(), 'someSecret', "$SubTestName - tests get of clientSecret");
    ok($CredentialDTO->isInTestMode(), "$SubTestName - tests if is in test mode");
    is($CredentialDTO->merchantClientId(), 'merchantID', "$SubTestName - tests get of merchantID");
}
#----------------------------------------------------------------------------------------
sub test_Error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createExperienceProfileId_Repository();
    my $PaymentMethodPaypalPlus = 'errorProducer';
    my $hResponse = {};

    throws_ok(sub {$Repository->buildRequest($PaymentMethodPaypalPlus)}, qr/PARAMETER_NO_OBJECT/, "$SubTestName - test if the Data could be collected");
    throws_ok(
        sub {$Repository->buildResponse($hResponse)},
        qr/PARAMETER_MISSING: Missing parameter ExperienceProfileId in call/, ## no critic (ProhibitComplexRegexes)
        "$SubTestName - test if Exepction is thrown"
    );
    throws_ok(sub {$Repository->writeResponse($PaymentMethodPaypalPlus);}, qr/PARAMETER_NO_OBJECT/, "$SubTestName - test if Exepction is thrown");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPaypalPlus {
    my $self = shift;
    my ($Shop) = @_;
    my $hAttributes = {
        'Site' => $Shop,
        'Shop' => $Shop,
        'MerchantClientId' => 'merchantID',
        'PaymentType' => $self->_createPaymentType(),
    };
    my $PaymentMethodPaypalPlus = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );
    return $PaymentMethodPaypalPlus;
}
#----------------------------------------------------------------------------------------
sub _createPaymentType {
    my $self = shift;
    my $hAttributes = {
        'ProviderClientId' => 'someId',
        'ProviderClientSecret' => 'someSecret',
        'PayPalPartnerAttributionId' => 'MyAttributionId',
    };
    my $PaymentType = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );
    return $PaymentType;
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'IsInternalTestShop' => 0,
        'NameOrAlias' => 'MyShopName',
        'LanguageID' => 2,
        'ProtocolAndServerSSL' => 'http://public',
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );

    return $Shop;
}
#----------------------------------------------------------------------------------------
sub _createExperienceProfileId_Repository {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
               'DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Repository',
               []
           );
    $Mockify->addMockWithReturnValueAndParameterCheck('_GetInfoLanguage', {'Code2' => 'de'}, [{'int' => 1}]);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();