#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::MerchantOnboardingURLBuilder
# base         base DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  Unit test for MerchantOnboardingURLBuilder
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::MerchantOnboardingURLBuilder;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder;
use URI;
use URI::QueryParam;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;
use strict;

sub test {
    my $self = shift;
    $self->test_getOnboardingURL_Live();
    $self->test_getOnboardingURL_Test();
    $self->test_getOnboardingURL_Error();
    done_testing();
}

sub test_getOnboardingURL_Live {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInternalTestShop = 0;
    my $ProviderPayPalAccountId = 'id';
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus($IsInternalTestShop, $self->_createPaymentType($ProviderPayPalAccountId));
    my $URLBuilder = DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder->new();
    my $ReturnURL = 'http://return.url';

    my $WasSuccessful = $URLBuilder->buildOnboardingURL($PaymentMethodPayPalPlus, $ReturnURL);
    my $URL = $URLBuilder->getOnboardingURL();

    is($WasSuccessful,1,"$SubTestName - Test if buildOnboardingURL was successful");
    my $hExpectedQuery = {
        'channelId' => 'partner',
        'partnerId' => 'id',
        'productIntentId' => 'ppplus',
        'displayMode' => 'regular',
        'integrationType' => 'TO',
        'features' => 'PAYMENT',
        'returnToPartnerUrl' => $ReturnURL,
        'tracking_id' => 'uniqueshopid',
        'partnerClientId' => 'clientid',
        'showPermissions' => 'TRUE',
    };
    my $URI = URI->new($URL);
    my %Query = $URI->query_form();
    is_deeply(\%Query, $hExpectedQuery, "$SubTestName - Test if query of URL is right");
    $URI->query_form({}); #this line removes queryparams
    is($URI,'http://liveurl.de',"$SubTestName - Test if onboarding URL is correct");
}

sub test_getOnboardingURL_Test {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInternalTestShop = 1;
    my $ProviderPayPalAccountId = 'id';
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus($IsInternalTestShop, $self->_createPaymentType($ProviderPayPalAccountId));
    my $URLBuilder = DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder->new();
    my $ReturnURL = 'http://return.url';

    my $WasSuccessful = $URLBuilder->buildOnboardingURL($PaymentMethodPayPalPlus, $ReturnURL);
    my $URL = $URLBuilder->getOnboardingURL();

    is($WasSuccessful,1,"$SubTestName - Test if buildOnboardingURL was successful");
    my $hExpectedQuery = {
        'channelId' => 'partner',
        'partnerId' => 'id',
        'productIntentId' => 'ppplus',
        'displayMode' => 'regular',
        'integrationType' => 'TO',
        'features' => 'PAYMENT',
        'returnToPartnerUrl' => $ReturnURL,
        'tracking_id' => 'uniqueshopid',
        'partnerClientId' => 'clientid-sandbox',
        'showPermissions' => 'TRUE',
    };
    my $URI = URI->new($URL);
    my %Query = $URI->query_form();
    is_deeply(\%Query, $hExpectedQuery, "$SubTestName - Test if query of URL is right");
    $URI->query_form({}); #this line removes queryparams
    is($URI,'http://sandboxurl.de',"$SubTestName - Test if onboarding URL is correct");
}

sub test_getOnboardingURL_Error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInternalTestShop = 1;
    my $ProviderPayPalAccountId = undef;
    my $PaymentMethodPayPalPlus = $self->_createPaymentMethodPayPalPlus($IsInternalTestShop, $self->_createPaymentType($ProviderPayPalAccountId));
    my $URLBuilder = DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder->new();
    my $ReturnURL = 'http://return.url';

    my $WasSuccessful = $URLBuilder->buildOnboardingURL($PaymentMethodPayPalPlus, $ReturnURL);
    my $Error = $URLBuilder->getError();

    is($WasSuccessful,0,"$SubTestName - Test if buildOnboardingURL was not successful");
    is($Error,'ProviderPayPalAccountError',"$SubTestName - Test if right Error was returned");
}

sub _createPaymentMethodPayPalPlus {
    my $self = shift;
    my ($IsInternalTestShop, $PaymentType) = @_;

    my $hAttributes = {
        'PaymentType' => $PaymentType,
        'Shop' => $self->_createShop($IsInternalTestShop),
    };
    my $PaymentMethodPayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );

    return $PaymentMethodPayPalPlus;
}

sub _createShop {
    my $self = shift;
    my ($IsInternalTestShop) = @_;

    my $hAttributes = {
        'IsInternalTestShop' => $IsInternalTestShop,
        'GUID' => 'uniqueshopid',
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );

    return $Shop;
}

sub _createPaymentType {
    my $self = shift;
    my ($ProviderPayPalAccountId) = @_;

    my $hAttributes = {
        'ProviderPayPalAccountId' => $ProviderPayPalAccountId,
        'ProviderClientId' => 'clientid',
        'ProviderClientIdSandbox' => 'clientid-sandbox',
        'MerchantOnboardingURLSandbox' => 'http://sandboxurl.de',
        'MerchantOnboardingURLLive' => 'http://liveurl.de',
    };
    my $PaymentType = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );

    return $PaymentType;
}

__PACKAGE__->runScript();