#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UserRedirect
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::PayPalPlus::API::UserRedirect
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UserRedirect;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::PayPalPlus::API::UserRedirect;
use DE_EPAGES::Test::Mock::Mockery;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::More;
use DE_EPAGES::PayPalPlus::API::Constants qw ( CLASSIC_CHECKOUT MULTISTEP_CHECKOUT );
use strict;

sub test {
    my $self = shift;

    $self->test_redirectToBasket_MultistepAndMobile();
    $self->test_redirectToBasket_ClassicAndMobile();
    $self->test_redirectToBasket_ClassicAndNotMobile();
    $self->test_redirectToBasket_MultistepAndNotMobile();
    $self->test_redirectToBasket_AdditionalParams();
    $self->test_redirectToPaymentSelectionPage();
    $self->test_redirectToPaymentSelectionPage_AdditionalParams();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_redirectToBasket_MultistepAndMobile {
    my $self = shift;
    my $SubTestName = (caller(0))[3];


    my $Mockery = $self->_createMocks( {
        'CheckOutType' => MULTISTEP_CHECKOUT,
        'IsMobileCheckout' => 1,
        'ExpectedParams' => {
            'ViewAction' => 'MobileSF-View',
        },
    } );

    lives_ok(
        sub { $Mockery->get('SUT')->redirectToBasket($Mockery->get('Servlet')); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}
#----------------------------------------------------------------------------------------
sub test_redirectToBasket_ClassicAndMobile {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks({
        'CheckOutType' => CLASSIC_CHECKOUT,
        'IsMobileCheckout' => 1,
        'ExpectedParams' => {
            'ViewAction' => 'MobileSF-View',
        },
    });
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToBasket($Mockery->get('Servlet')); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'),1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}
#----------------------------------------------------------------------------------------
sub test_redirectToBasket_ClassicAndNotMobile {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks( {
        'CheckOutType' => CLASSIC_CHECKOUT,
        'IsMobileCheckout' => 0,
        'ExpectedParams' => {
            'ViewAction' => 'View',
        },
    } );
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToBasket($Mockery->get('Servlet')); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}
#----------------------------------------------------------------------------------------
sub test_redirectToBasket_MultistepAndNotMobile {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks( {
        'CheckOutType' => MULTISTEP_CHECKOUT,
        'IsMobileCheckout' => 0,
        'ExpectedParams' => { },
    } );
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToBasket($Mockery->get('Servlet')); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 0, "$SubTestName - check if the redirect was not called one time");
}
#----------------------------------------------------------------------------------------
sub test_redirectToBasket_AdditionalParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks( {
        'CheckOutType' => CLASSIC_CHECKOUT,
        'IsMobileCheckout' => 0,
        'ExpectedParams' => { 'ViewAction' => 'View', 'additionalParam' => 'value' },
    } );
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToBasket($Mockery->get('Servlet'), { 'additionalParam' => 'value' }); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}

#----------------------------------------------------------------------------------------
sub test_redirectToPaymentSelectionPage{
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks( {
        'CheckOutType' => MULTISTEP_CHECKOUT,
        'IsMobileCheckout' => 0,
        'ExpectedParams' => { 'ViewAction' => 'ViewMultiCheckoutPayment',},
    } );
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToPaymentSelectionPage($Mockery->get('Servlet')); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}
#----------------------------------------------------------------------------------------
sub test_redirectToPaymentSelectionPage_AdditionalParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks( {
        'CheckOutType' => MULTISTEP_CHECKOUT,
        'IsMobileCheckout' => 0,
        'ExpectedParams' => { 'ViewAction' => 'ViewMultiCheckoutPayment', 'additionalParam' => 'value' },
    } );
    lives_ok(
        sub { $Mockery->get('SUT')->redirectToPaymentSelectionPage($Mockery->get('Servlet'), { 'additionalParam' => 'value' }); },
        "$SubTestName - triggered the redirectToBasket"
    );
    is(WasCalled($Mockery->get('Servlet'), 'redirect'), 1, "$SubTestName - check if the redirect was called one time");

    my $aParameters = GetParametersFromMockifyCall($Mockery->get('SUT'), '_BuildShopUrl');
    my $hOptions = $aParameters->[2];
    my $hExpectedOptions = {
        'Type' => 'sf',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    is_deeply($hOptions, $hExpectedOptions, "$SubTestName - Test if options are as expected");
}
#---------------------------------------------------------------------------
#  helpers
#---------------------------------------------------------------------------

sub _createMocks {
    my $self = shift;
    my ($hSetup) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();
    #----------------------------------------------------------------------------------------
    $Mockery->register('SUT', ['Logger'], sub {
        my ($Logger) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::UserRedirect',[]);
        $Mockify->mock('_BuildShopUrl','https://super.dummy.com/secBla',[
            {'object' => 'DE_EPAGES::Order::API::Object::Basket'},
            {'hashref' => $hSetup->{'ExpectedParams'}},
            'hashref',
            {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
        ]);
        $Mockify->mock('_createLogger', $Logger);
        return $Mockify->getMockObject();
      }
    );
    #----------------------------------------------------------------------------------------
    $Mockery->register('Logger',[], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::Logger',[]);
        $Mockify->mock('logPayment', undef, [
            {'string' => 'redirect to basket page after customer logged in'},
            {'hashref' => { 'RedirectUrl' => 'https://super.dummy.com/secBla' }}
        ]);
        return $Mockify->getMockObject();
    });
    #----------------------------------------------------------------------------------------
    $Mockery->register('Servlet', ['Basket'], sub {
        my ($Basket) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $Basket );
        $Mockify->addVar('IsMobile',$hSetup->{'IsMobileCheckout'});
        $Mockify->addParam('QUERY_STRING',$hSetup->{'queryStringCheckout'});
        $Mockify->mock('redirect', undef, [{'string'=>'https://super.dummy.com/secBla'}]);
        my $ABC = $Mockify->getMockObject();
        return $ABC;
    });
    #----------------------------------------------------------------------------------------
    $Mockery->register('Basket', ['Shop', 'LineItemContainer'], sub {
        my ($Shop, $LineItemContainer) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket'
            ,{
                'LineItemContainer' => $LineItemContainer,
                'Site' => $Shop,
            });
        return $Mockify->getMockObject();
    });
    #----------------------------------------------------------------------------------------
    $Mockery->register('LineItemContainer', ['LineItemPayment'], sub {
        my ($LineItemPayment) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer'
            ,{
                'Payment' => $LineItemPayment,
            });
        return $Mockify->getMockObject();
    });
    #----------------------------------------------------------------------------------------
    $Mockery->register('LineItemPayment', [], sub {
        my ($PaymentMethodPayPalPlus) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus'
            ,{ });
        return $Mockify->getMockObject();
    });
    #----------------------------------------------------------------------------------------
    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop'
            ,{
                'OrderProcess' => $hSetup->{'CheckOutType'}, # 1=multistep 0=classic
            });
        return $Mockify->getMockObject();
    });
    #----------------------------------------------------------------------------------------
    return $Mockery->assemble();
}

__PACKAGE__->runScript();