#==================================================================================================
# package      DE_EPAGES::SIA::t::PaymentMethodSIA
# state        public
#--------------------------------------------------------------------------------------------------
# description  Integration test for DE_EPAGES::SIA::API::Object::PaymentMethodSIA
#==================================================================================================
package DE_EPAGES::SIA::t::PaymentMethodSIA;
use base DE_EPAGES::Shop::API::Test;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::SIA::API::Constants qw( TRANS_STATUS_PENDING TRANS_STATUS_ACCEPTED TRANS_STATUS_DECLINED);
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_usableAtStoreFront_regular_test_config();

    $self->reset();
    $self->test_usableAtStoreFront_regular_live_config();

    $self->reset();
    $self->test_usableAtStoreFront_missing_params();

    $self->reset();
    $self->test_existsCurrentRegistrationTransaction();

    $self->reset();
    $self->test_hasPendingCurrentTransaction();

    $self->reset();
    $self->test_hasAcceptedCurrentTransaction();

    $self->reset();
    $self->test_hasDeclinedCurrentTransaction();

    done_testing();
    return;
}

sub reset {
    my $self = shift;

    # reset manipulated data
    $self->cleanup();
    $self->init();

    return;
}
sub init {
    my $self = shift;

    $self->SUPER::init;

    my $Shop = $self->shop;

    $self->increaseFeatureLimit( $Shop, 'PaymentMethods' );
    $self->increaseFeatureLimit( $Shop, 'SIA' );
    $self->createPaymentMethodSia($Shop);
    $self->createShopAddress($Shop);

    return;
}

sub cleanup {
    my $self = shift;

    $self->SUPER::cleanup;

    return;
}


sub test_existsCurrentRegistrationTransaction {
    my $self = shift;

    note 'test_existsCurrentRegistrationTransaction';
    my $PaymentMethodSia = $self->paymentMethod;

    my $existsCurrentRegistrationTransaction = $PaymentMethodSia->existsCurrentRegistrationTransaction();
    is($existsCurrentRegistrationTransaction, 0, 'initial payment has no current transaction');

    my $hRegistrationTransaction    = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction     = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $existsCurrentRegistrationTransaction   = $PaymentMethodSia->existsCurrentRegistrationTransaction();
    is($existsCurrentRegistrationTransaction, 1, 'after providing transdata payment has current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_ACCEPTED});
    $existsCurrentRegistrationTransaction   = $PaymentMethodSia->existsCurrentRegistrationTransaction();
    is($existsCurrentRegistrationTransaction, 1, 'changing transstatus to accepted - payment has no current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $existsCurrentRegistrationTransaction   = $PaymentMethodSia->existsCurrentRegistrationTransaction();
    is($existsCurrentRegistrationTransaction, 1, 'changing transstatus to declined - payment has no current transaction');

    return;

}


sub test_hasPendingCurrentTransaction {
    my $self = shift;

    note 'test_hasPendingCurrentTransaction';
    my $PaymentMethodSia = $self->paymentMethod;

    my $hasPendingCurrentTransaction = $PaymentMethodSia->hasPendingCurrentTransaction();
    is($hasPendingCurrentTransaction, 0, 'initial payment has no current transaction');

    my $hRegistrationTransaction    = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction     = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $hasPendingCurrentTransaction   = $PaymentMethodSia->hasPendingCurrentTransaction();
    is($hasPendingCurrentTransaction, 1, 'after providing transdata payment has current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_ACCEPTED});
    $hasPendingCurrentTransaction   = $PaymentMethodSia->hasPendingCurrentTransaction();
    is($hasPendingCurrentTransaction, 0, 'changing transstatus to accepted - payment has no current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $hasPendingCurrentTransaction   = $PaymentMethodSia->hasPendingCurrentTransaction();
    is($hasPendingCurrentTransaction, 0, 'changing transstatus to declined - payment has no current transaction');

    return;

}

sub test_hasAcceptedCurrentTransaction {
    my $self = shift;

    note 'test_hasAcceptedCurrentTransaction';

    my $PaymentMethodSia = $self->paymentMethod;

    my $hasAcceptedCurrentTransaction = $PaymentMethodSia->hasAcceptedCurrentTransaction();
    is($hasAcceptedCurrentTransaction, 0, 'initial payment has no current transaction');

    my $hRegistrationTransaction    = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction     = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $hasAcceptedCurrentTransaction   = $PaymentMethodSia->hasAcceptedCurrentTransaction();
    is($hasAcceptedCurrentTransaction, 0, 'after providing transdata payment has current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_ACCEPTED});
    $hasAcceptedCurrentTransaction   = $PaymentMethodSia->hasAcceptedCurrentTransaction();
    is($hasAcceptedCurrentTransaction, 1, 'changing transstatus to accepted - payment has no current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $hasAcceptedCurrentTransaction   = $PaymentMethodSia->hasAcceptedCurrentTransaction();
    is($hasAcceptedCurrentTransaction, 0, 'changing transstatus to declined - payment has no current transaction');

    return;

}

sub test_hasDeclinedCurrentTransaction {
    my $self = shift;

    note 'test_hasDeclinedCurrentTransaction';

    my $PaymentMethodSia = $self->paymentMethod;

    my $hasDeclinedCurrentTransaction = $PaymentMethodSia->hasDeclinedCurrentTransaction();
    is($hasDeclinedCurrentTransaction, 0, 'initial payment has no current transaction');

    my $hRegistrationTransaction    = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction     = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $hasDeclinedCurrentTransaction   = $PaymentMethodSia->hasDeclinedCurrentTransaction();
    is($hasDeclinedCurrentTransaction, 0, 'after providing transdata payment has current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_ACCEPTED});
    $hasDeclinedCurrentTransaction   = $PaymentMethodSia->hasDeclinedCurrentTransaction();
    is($hasDeclinedCurrentTransaction, 0, 'changing transstatus to accepted - payment has no current transaction');

    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $hasDeclinedCurrentTransaction   = $PaymentMethodSia->hasDeclinedCurrentTransaction();
    is($hasDeclinedCurrentTransaction, 1, 'changing transstatus to declined - payment has no current transaction');

    return;

}


# Test Config
sub test_usableAtStoreFront_regular_test_config {
    my $self = shift;

    note 'test_usableAtStoreFront_regular_test_config';

    my $PaymentMethodSia = $self->paymentMethod;

    my $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    my $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 1, 'test payment can be set to 1 independent from registration state' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_PENDING});
    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 1, 'test payment can be set to 1 if pending merchant registration' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    $RegistrationTransaction = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 1, 'test payment can be set to 1 if declined merchant registration' );

    return;
}

# Live Config
sub test_usableAtStoreFront_regular_live_config {
    my $self = shift;

    note 'test_usableAtStoreFront_regular_live_config';

    my $PaymentMethodSia = $self->paymentMethod;
    $PaymentMethodSia->set({'IsInTestMode' => 0});

    my $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    my $RegistrationTransaction = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_ACCEPTED});
    my $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 1, 'live payment can be set to 1 if accepted registration' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    $RegistrationTransaction = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_PENDING});
    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 if merchant pending' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');
    $RegistrationTransaction = $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);
    $RegistrationTransaction->set({'TransStatus' => TRANS_STATUS_DECLINED});
    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();

    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 if merchant declined' );

    return;
}

sub test_usableAtStoreFront_missing_params {
    my $self = shift;

    note 'test_usableAtStoreFront_missing_params';

    my $PaymentMethodSia = $self->paymentMethod;

    my $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('','','','');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    my $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'test payment can\'t be set to 1 - missing all params' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','','','');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'test payment can\'t be set to 1 - missing most params' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','','');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'test payment can\'t be set to 1 - missing terminal number and marchant tax' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    # Switching to Live to also text this configuration
    $PaymentMethodSia->set({'IsInTestMode' => 0});

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 - missing merchant tax' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','','98798');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 - missing terminal number' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','','123','98798');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 - missing merchant id' );

    $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('','123123','123','98798');
    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $usableAtStoreFront = $PaymentMethodSia->usableAtStorefront();
    is( $usableAtStoreFront, 0, 'live payment can\'t be set to 1 - missing application payment network' );

    return;
}

sub increaseFeatureLimit {
    my $self = shift;
    my ( $Shop, $FeatureName ) = @_;

    my $CurrentFeatureValue = $Shop->featureCurrentValue($FeatureName);
    my $MaxFeatureValue     = $Shop->featureMaxValue($FeatureName);

    if ( $CurrentFeatureValue >= $MaxFeatureValue ) {
        $Shop->featureMaxValue( $FeatureName, $CurrentFeatureValue + 1, 1 );
    }

    return;
}

sub createPaymentMethodSia {
    my $self = shift;
    my ($Shop) = @_;

    my $PaymentType        = LoadObjectByPath('/PaymentTypes/SIA');
    my $hPaymentMethodInfo = {
        'Alias'     => 'SIA',
        'Position'  => 9999,
        'IsDefault' => 1,
    };

    my $PaymentMethod = $PaymentType->newPaymentMethod( $Shop, $hPaymentMethodInfo );
    $self->paymentMethod($PaymentMethod);

    return;
}

sub paymentMethod {
    my $self = shift;
    my ($PaymentMethod) = @_;

    $self->{'PaymentMethod'} = $PaymentMethod if defined $PaymentMethod;

    return $self->{'PaymentMethod'};
}

sub createShopAddress {
    my $self = shift;
    my ($Shop) = @_;

    my $ShopAddress = $Shop->get('Address');

    $ShopAddress->set({
            'City'      => 'TestCity',
            'CountryID' => '276',
            'LastName'  => 'LastName',
        });

    return;
}

__PACKAGE__->runScript();
1;
