#==================================================================================================
# package      DE_EPAGES::SIA::t::SFTPHandler
# state        public
#--------------------------------------------------------------------------------------------------
# description  Integration test for DE_EPAGES::SIA::API::Registration::SFTPHandler
#==================================================================================================
package DE_EPAGES::SIA::t::SFTPHandler;
use base DE_EPAGES::Shop::API::Test;

use DE_EPAGES::Core::API::File qw ( ExistsFile DeleteFile );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::SIA::API::Registration::SFTPHandler;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_addDataToUploadFile_success();

    done_testing();
    return;
}

sub init {
    my $self = shift;

    $self->SUPER::init;

    my $Shop = $self->shop;

    $self->increaseFeatureLimit( $Shop, 'PaymentMethods' );
    $self->increaseFeatureLimit( $Shop, 'SIA' );
    $self->createPaymentMethodSia($Shop);
    $self->createShopAddress($Shop);

    my $SiaConfig = $self->paymentMethod->getConfig();

    $self->createSftpHandler($SiaConfig);

    return;
}

sub cleanup {
    my $self = shift;

    $self->SUPER::cleanup;

    DeleteFile( $self->registrationFile );

    return;
}

sub test_addDataToUploadFile_success {
    my $self = shift;

    my $SftpHandler      = $self->sftpHandler;
    my $PaymentMethodSia = $self->paymentMethod;

    # Create registration transaction that will be written to a file.
    my $hRegistrationTransaction = $PaymentMethodSia->provideRegistrationTransactionData('Adyen','123123','123','98798');

    $PaymentMethodSia->saveRegistrationTransaction($hRegistrationTransaction);

    $SftpHandler->addDataToUploadFile($PaymentMethodSia);

    my $FilePath = $SftpHandler->FilePath();
    $self->registrationFile($FilePath);

    ok( ExistsFile($FilePath), 'Registration file exists' );

    $SftpHandler->uploadFile();

    ok($SftpHandler->existsRemoteFile(),'Remote registration file exists');

    $SftpHandler->deleteFile();

    ok(!ExistsFile($FilePath), 'Registration file deleted' );
    return;
}

sub increaseFeatureLimit {
    my $self = shift;
    my ( $Shop, $FeatureName ) = @_;

    my $CurrentFeatureValue = $Shop->featureCurrentValue($FeatureName);
    my $MaxFeatureValue     = $Shop->featureMaxValue($FeatureName);

    if ( $CurrentFeatureValue >= $MaxFeatureValue ) {
        $Shop->featureMaxValue( $FeatureName, $CurrentFeatureValue + 1, 1 );
    }

    return;
}

sub createPaymentMethodSia {
    my $self = shift;
    my ($Shop) = @_;

    my $PaymentType        = LoadObjectByPath('/PaymentTypes/SIA');
    my $hPaymentMethodInfo = {
        'Alias'     => 'SIA',
        'Position'  => 9999,
        'IsDefault' => 1,
    };

    my $PaymentMethod = $PaymentType->newPaymentMethod( $Shop, $hPaymentMethodInfo );
    $self->paymentMethod($PaymentMethod);

    return;
}

sub createSftpHandler {
    my $self = shift;
    my ($SiaConfig) = @_;

    my $SftpHandler = DE_EPAGES::SIA::API::Registration::SFTPHandler->new( 'SiaConfig' => $SiaConfig );
    $SftpHandler->createRegistrationFile();
    $self->sftpHandler($SftpHandler);

    return;
}

sub paymentMethod {
    my $self = shift;
    my ($PaymentMethod) = @_;

    $self->{'PaymentMethod'} = $PaymentMethod if defined $PaymentMethod;

    return $self->{'PaymentMethod'};
}

sub sftpHandler {
    my $self = shift;
    my ($SftpHandler) = @_;

    $self->{'SftpHandler'} = $SftpHandler if defined $SftpHandler;

    return $self->{'SftpHandler'};
}

sub registrationFile {
    my $self = shift;
    my ($RegistrationFile) = @_;

    $self->{'RegistrationFile'} = $RegistrationFile if defined $RegistrationFile;

    return $self->{'RegistrationFile'};
}

sub createShopAddress {
    my $self = shift;
    my ($Shop) = @_;

    my $ShopAddress = $Shop->get('Address');

    $ShopAddress->set({
            'City'      => 'TestCity',
            'CountryID' => '276',
            'LastName'  => 'LastName',
        });

    return;
}

__PACKAGE__->runScript();
1;
