#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::Cancel
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Request::Cancel
#==================================================================================================
package DE_EPAGES::SIA::t::unit::Cancel;
use base DE_EPAGES::Core::API::Test;

use feature                                 qw (switch);
use DE_EPAGES::Core::API::PerlTools         qw (isValid);
use DE_EPAGES::SIA::t::unit::MockupObjects  qw (CreateMockedLineItemPayment
                                                TEST_MERCHANT_FUC
                                                TEST_TERMINAL_NUMBER
                                                TEST_CURRENCY_ID
                                                TEST_TRANS_ID
                                                TEST_REMOTE_ADDRESS);
use DE_EPAGES::SIA::API::Constants          qw (PROTOCOL_VERSION
                                                PROTOCOL_VERSION_LENGTH
                                                ID_TERMINAL_LENGTH
                                                CLIENT_ENTITY_ID_LENGTH
                                                MERCHANT_FUC_LENGTH
                                                USER_ID_LENGTH
                                                TRANSACTION_TYPE_LENGTH
                                                OPC_TOKEN_ID_LENGTH
                                                WEB_REQUEST_ID_LENGTH
                                                LANGUAGE_LENGTH
                                                TRANSACTION_CURRENCY_LENGTH);
use DE_EPAGES::SIA::API::Object::LineItemPaymentSIA;
use Test::More;
use JSON;
use strict;

## no critic(ProhibitMagicNumbers)
sub test {
    my $self = shift;

    $self->test_getPaymentCancelRequest();

    done_testing();
}

sub test_getPaymentCancelRequest {
    my $self = shift;

    my $MockedLineItemPaymentSIA    = CreateMockedLineItemPayment();
    my $JsonRequestData             = $MockedLineItemPaymentSIA->getPaymentCancelRequest()->getRequestData();
    my $hRequestData                = decode_json($JsonRequestData);

    $self->testField('idVersion'    ,$hRequestData, PROTOCOL_VERSION        ,PROTOCOL_VERSION_LENGTH);
    $self->testField('idTerminal'   ,$hRequestData, TEST_TERMINAL_NUMBER    ,ID_TERMINAL_LENGTH);
    $self->testField('idEntidad'    ,$hRequestData, 'ACEN'                  ,CLIENT_ENTITY_ID_LENGTH);
    $self->testField('idComercio'   ,$hRequestData, TEST_MERCHANT_FUC       ,MERCHANT_FUC_LENGTH);
    $self->testField('idUsuario'    ,$hRequestData, ''                      ,USER_ID_LENGTH);
    $self->testField('iDTM'         ,$hRequestData, '1420'                  ,TRANSACTION_TYPE_LENGTH);
    $self->testField('opcToken'     ,$hRequestData, '01'                    ,OPC_TOKEN_ID_LENGTH);
    $self->testField('idPedido'     ,$hRequestData, TEST_TRANS_ID           ,WEB_REQUEST_ID_LENGTH);
    $self->testField('codMon'       ,$hRequestData, TEST_CURRENCY_ID        ,TRANSACTION_CURRENCY_LENGTH);
    $self->testField('idioma'       ,$hRequestData, 'de'                    ,LANGUAGE_LENGTH);
    $self->testField('timeTrans'    ,$hRequestData, '2015-12-08T07:24:31.000 Europe/Berlin');
    $self->testField('impTrans'     ,$hRequestData, '8000');
    $self->testField('idServicio'   ,$hRequestData, '');
    $self->testField('ipOrigen'     ,$hRequestData, TEST_REMOTE_ADDRESS);

    # TODO: Activate test for firm token.
    #$self->testValue('firmToken',$hRequestData, 'todo');

    $self->testField('token'        ,$hRequestData, '');
    $self->testField('desToken'     ,$hRequestData, '');

    # specific cancel data
    $self->testField('fechProOri'   ,$hRequestData, '150629102556');
    $self->testField('numSIAOri'    ,$hRequestData, '2541');
    $self->testField('sesionComercio',$hRequestData,'');

    return;
}

sub testField {
    my $self = shift;
    my ($FieldName,$hData,$ExpectedValue,$Length) = @_;

    if (isValid $Length) {
        my $Blanks = ' ' x ($Length - length($ExpectedValue)) ;
        is($hData->{$FieldName},$ExpectedValue.$Blanks, $FieldName.' Value');
    }
    else {
        is($hData->{$FieldName},$ExpectedValue, $FieldName.' Value');
    }

    return;
}

__PACKAGE__->runScript();

1;
