#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::FixedString
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Types::FixedString
#==================================================================================================
package DE_EPAGES::SIA::t::unit::FixedString;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::SIA::API::Types::FixedString;
use Test::Exception;
use Test::More;

use strict;

## no critic qw(ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_constructor_undefParams();
    $self->test_constructor_invalidType();
    $self->test_constructor_typeConversion();
    $self->test_constructor_success();
    $self->test_constructor_aroundBuildargs();

    done_testing();
    return;
}

sub test_constructor_undefParams {
    my $self = shift;

    my $FixedString = DE_EPAGES::SIA::API::Types::FixedString->new();
    ok( $FixedString, 'constructor undef params' );

    return;
}

sub test_constructor_invalidType {
    my $self = shift;

    throws_ok(
        sub {
            DE_EPAGES::SIA::API::Types::FixedString->new( value => 'Test', size => '10a' );
        },
        qr/type/,
        'constructor invalid type'
    );

    return;
}

sub test_constructor_typeConversion {
    my $self = shift;

    my $OriginalString = 'Test';
    my $FixedString = DE_EPAGES::SIA::API::Types::FixedString->new( value => 'Test', size => '10' );
    is( $FixedString->value,          $OriginalString,            'FixedString value' );
    is( $FixedString->formattedValue, $OriginalString . '      ', 'FixedString formattedValue' );

    return;
}

sub test_constructor_success {
    my $self = shift;

    my $OriginalString = 'Test';
    my $FixedString = DE_EPAGES::SIA::API::Types::FixedString->new( value => 'Test', size => 10 );
    is( $FixedString->value,          $OriginalString,            'FixedString value' );
    is( $FixedString->formattedValue, $OriginalString . '      ', 'FixedString formattedValue' );

    return;
}

sub test_constructor_aroundBuildargs {
    my $self = shift;

    my $OriginalString = 'Test';
    my $FixedString = DE_EPAGES::SIA::API::Types::FixedString->new( 'Test', 10 );
    is( $FixedString->value,          $OriginalString,            'FixedString value' );
    is( $FixedString->formattedValue, $OriginalString . '      ', 'FixedString formattedValue' );

    return;
}

__PACKAGE__->runScript();
1;
