#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::MerchantRegistrationData
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Types::MerchantRegistrationData
#==================================================================================================
package DE_EPAGES::SIA::t::unit::MerchantRegistrationData;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::SIA::API::Tools              qw (GetToolsInstance);
use DE_EPAGES::SIA::t::unit::MockupObjects  qw (CreateMockedPaymentMethod);
use DE_EPAGES::Core::API::Error             qw (ExistsError);
use DE_EPAGES::SIA::API::Constants          qw (
    DEFAULTREQUESTCODE
    DEFAULTREQUESTSUBCODE
    PRIVATECARDFLAG
    DCCFLAG
    TAXFREEFLAG
    KINDOFLOCATION
    REGIONCODE
    RSAVERIFICATION
    RECURRENTADMITTED
    THREEDSECURECONFIGURATION
    RECURRENTMAXIMUM
    TOKENDAYSOFEXPIRATION
    QUARANTINETIME
    CAPTCHACODE
    TIMESTAMPTIME
    DAYSHISTORY
    GETISSUERBANKCOUNTRY
    GETCOUNTRYIPORIGIN
    ROUTINGLEVEL
    CANCELLATIONOPTIONS
    REFUNDOPTIONS
    REFUNDDAYS
    );
use DE_EPAGES::SIA::API::Types::MerchantRegistrationData;
use Test::More;

use strict;

## no critic (ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_constructor_params_min_default();
    $self->test_constructor_params_error();

    done_testing();
    return;
}

sub test_constructor_params_error {
    my $self = shift;
    eval {
        DE_EPAGES::SIA::API::Types::MerchantRegistrationData->new();
    };
    if ( ExistsError() ) {
        pass('Error was thrown when trying to create Merchantregistration without data');
    }
    else {
        fail('No Error was thrown when trying to create Merchantregistration without data');
    }

    return;
}

# test short and long values aswell as normal
sub test_constructor_params_min_default {
    my $self = shift;

    my $CurrentDate            = GetToolsInstance()->getCurrentDate();

    my $MerchantRegistrationData = DE_EPAGES::SIA::API::Types::MerchantRegistrationData->new(
        'Alias'                     => 'TestAlias',
        'Payment'                   => CreateMockedPaymentMethod(),
        'IMCompany'                 => 1234,
        'WorkOrderID'               => 'WOID',
        'CompanyCode'               => 'Code',
        'HoldingCode'               => 'HCod',
        'MerchantName'              => 'Name',
        'City'                      => 'City',
        'Country'                   => 276,
        'ApplicationPaymentNetwork' => '120001',
        'MerchantLanguage'          => 'de',
        'Currency'                  => 'EUR',
        'MerchantTaxID'             => '9879',
        'MerchantID'                => '123123123123123',
        'MerchantFUC'               => '004123123123123123',
        'AcquiringFinancialEntity'  => 'ADYN',
        'WorkOrderDate'             => $CurrentDate,
    );

    is( $MerchantRegistrationData->IMCompany(),     1234,                                   'IMCompany' );
    is( $MerchantRegistrationData->WorkOrderID(),   'WOID       ',                          'WorkOrderID' );
    is( $MerchantRegistrationData->RequestCode(),   DEFAULTREQUESTCODE,                     'RequestCode' );
    is( $MerchantRegistrationData->RequestSubCode(),DEFAULTREQUESTSUBCODE,                  'RequestSubCode' );
    is( $MerchantRegistrationData->EntityCode(),    'ACEN',                                 'EntityCode' );
    is( $MerchantRegistrationData->CompanyCode(),   'Code',                                 'CompanyCode' );
    is( $MerchantRegistrationData->HoldingCode(),   'HCod',                                 'HoldingCode' );
    is( $MerchantRegistrationData->MerchantFUC(),   '004123123123123',                      'MerchantFUC' );
    is( $MerchantRegistrationData->MerchantName(),  'Name                          ',       'MerchantName' );
    is( $MerchantRegistrationData->City(),          'City                               ',  'City' );
    is( $MerchantRegistrationData->Country(),       276,                                    'Country' );
    is( $MerchantRegistrationData->WorkOrderDate(), $CurrentDate,                           'WorkOrderDate' );
    is( $MerchantRegistrationData->MerchantTaxID(), '9879                  ',               'MerchantTaxID' );
    is( $MerchantRegistrationData->MerchantLanguage(),          'de',                       'MerchantLanguage' );
    is( $MerchantRegistrationData->Currency(),                  'EUR',                      'Currency' );
    is( $MerchantRegistrationData->ApplicationPaymentNetwork(), '120001',                   'ApplicationPaymentNetwork' );
    is( $MerchantRegistrationData->PrivateCardFlag(),           PRIVATECARDFLAG,            'PrivateCardFlag' );
    is( $MerchantRegistrationData->DccFlag(),                   DCCFLAG,                    'DccFlag' );
    is( $MerchantRegistrationData->TaxFreeFlag(),               TAXFREEFLAG,                'TaxFreeFlag' );
    is( $MerchantRegistrationData->MerchantID(),                '123123123123123',          'MerchantID' );
    is( $MerchantRegistrationData->KindOfLocation(),            KINDOFLOCATION,             'KindOfLocation' );
    is( $MerchantRegistrationData->RegionCode(),                REGIONCODE,                 'RegionCode');
    is( $MerchantRegistrationData->RsaVerification(),           RSAVERIFICATION,            'RsaVerification' );
    is( $MerchantRegistrationData->RecurrentAdmitted(),         RECURRENTADMITTED,          'RecurrentAdmitted' );
    is( $MerchantRegistrationData->ThreeDSecureConfiguration(), THREEDSECURECONFIGURATION,  'ThreeDSecureConfiguration' );
    is( $MerchantRegistrationData->RecurrentMaximum(),          RECURRENTMAXIMUM,           'RecurrentMaximum' );
    is( $MerchantRegistrationData->TokenDaysOfExpiration(),     TOKENDAYSOFEXPIRATION,      'TokenDaysOfExpiration' );
    is( $MerchantRegistrationData->QuarantineTime(),            QUARANTINETIME,             'QuarantineTime' );
    is( $MerchantRegistrationData->CaptchaOption(),             CAPTCHACODE,                'CaptchaOption' );
    is( $MerchantRegistrationData->TimeStampTime(),             TIMESTAMPTIME,              'TimeStampTime' );
    is( $MerchantRegistrationData->DaysHistory(),               DAYSHISTORY,                'DaysHistory' );
    is( $MerchantRegistrationData->GetIssuerBankCountry(),      GETISSUERBANKCOUNTRY,       'GetIssuerBankCountry' );
    is( $MerchantRegistrationData->GetCountryIPOrigin(),        GETCOUNTRYIPORIGIN,         'GetCountryIPOrigin' );
    is( $MerchantRegistrationData->RoutingLevel(),              ROUTINGLEVEL,               'RoutingLevel' );
    is( $MerchantRegistrationData->CancellationOptions(),       CANCELLATIONOPTIONS,        'CancellationOptions' );
    is( $MerchantRegistrationData->RefundOptions(),             REFUNDOPTIONS,              'RefundOptions' );
    is( $MerchantRegistrationData->RefundDays(),                REFUNDDAYS,                 'RefundDays' );
    is( $MerchantRegistrationData->AcquiringFinancialEntity(),  'ADYN',                     'AcquiringFinancialEntity' );

    return;
}

__PACKAGE__->runScript();
1;
