#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::PaymentMethodSIA
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Object::PaymentMethodSIA
#==================================================================================================
package DE_EPAGES::SIA::t::unit::PaymentMethodSIA;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::SIA::API::Object::LineItemPaymentSIA;
use DE_EPAGES::SIA::API::Types::MerchantRegistrationData;
use DE_EPAGES::SIA::t::unit::MockupObjects  qw (CreateMockedPaymentMethod);
use Test::Exception;
use Test::More;

use strict;

## no critic(ProhibitMagicNumbers)
sub test {
    my $self = shift;

    $self->getSaleRedirectURL();

    done_testing();
}

sub getSaleRedirectURL {
    my $self = shift;

    $self->test_TestRedirectURL();
    # TODO: Activate tests for live URLs.
    #$self->test_LiveRedirectURL();
    $self->test_getCurrentRegistrationTransaction_success();
    $self->test_getCurrentRegistrationTransaction_noRegistrationTransactions();
    $self->test_getCurrentRegistrationTransaction_transactionMissing();
    $self->test_addMerchantRegistrationData();
    $self->test_addMerchantRegistrationData_empty();
    $self->test_addMerchantRegistrationData_undef();
    return;
}
sub test_addMerchantRegistrationData_undef() {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    my $hResult = $MockedPaymentMethodSIA->provideRegistrationTransactionData(undef,undef,undef,undef);

    # Do not throw error - error will be thrown in insert because of moose object definition
    is($hResult->{'ApplicationPaymentNetwork'}  ,undef,'undef ApplicationPaymentNetwork correct');
    is($hResult->{'MerchantID'}                 ,undef,'undef MerchantID correct');
    is($hResult->{'TerminalNumber'}             ,undef,'undef TerminalNumber correct');
    is($hResult->{'MerchantTaxID'}              ,undef,'undef MerchantTaxID correct');

    return;
}

sub test_addMerchantRegistrationData_empty {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    my $hResult = $MockedPaymentMethodSIA->provideRegistrationTransactionData('','','','');

    # Do not throw error -> error will be thrown in insert because of moose object definition
    is($hResult->{'ApplicationPaymentNetwork'}  ,'','empty ApplicationPaymentNetwork correct');
    is($hResult->{'MerchantID'}                 ,'','empty MerchantID correct');
    is($hResult->{'TerminalNumber'}             ,'','empty TerminalNumber correct');
    is($hResult->{'MerchantTaxID'}              ,'','empty MerchantTaxID correct');

    return;
}

sub test_addMerchantRegistrationData {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    my $hResult = $MockedPaymentMethodSIA->provideRegistrationTransactionData('Adyen','123123123123123','12312312','9486948');

    is($hResult->{'ApplicationPaymentNetwork'}  ,'Adyen','ApplicationPaymentNetwork correct');
    is($hResult->{'MerchantID'}                 ,'123123123123123','MerchantID correct');
    is($hResult->{'TerminalNumber'}             ,'12312312','TerminalNumber correct');
    is($hResult->{'CompanyCode'}                ,'0001','CompanyCode correct');
    is($hResult->{'HoldingCode'}                ,'0001','HoldingCode correct');
    is($hResult->{'AcquiringFinancialEntity'}   ,'ADYN','AcquiringFinancialEntity correct');
    like($hResult->{'MerchantFUC'}              ,qr/004*/,'MerchantFUC correct');
    is($hResult->{'MerchantName'}               ,'TestFirstName TestLastName','merchant name correct');
    is($hResult->{'MerchantTaxID'}              ,'9486948','MerchantTaxID correct');
    is($hResult->{'City'}                       ,'TestCity','city correct');
    is($hResult->{'Country'}                    ,'276','country correct');
    is($hResult->{'MerchantLanguage'}           ,'de','language correct');
    ok($hResult->{'WorkOrderID'}                ,'WorkOrderID correct');

    return;
}

sub test_TestRedirectURL {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    my $RedirectURL = $MockedPaymentMethodSIA->getSaleRedirectURL();
    is($RedirectURL,'http://216.177.213.245:6990/servlet/ServletDirector','test sale redirect url');

    $RedirectURL = $MockedPaymentMethodSIA->getRefundRedirectURL();
    is($RedirectURL,'http://216.177.213.245:6982/Payment/refund','test refund redirect url');

    $RedirectURL = $MockedPaymentMethodSIA->getCancelRedirectURL();
    is($RedirectURL,'http://216.177.213.245:6982/Payment/cancel','test cancel redirect url');
    return;
}

sub test_LiveRedirectURL {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    $MockedPaymentMethodSIA->set({'IsInTestMode' => 0});

    my $RedirectURL = $MockedPaymentMethodSIA->getSaleRedirectURL();
    is($RedirectURL,'todo','live sale redirect url');

    $RedirectURL = $MockedPaymentMethodSIA->getRefundRedirectURL();
    is($RedirectURL,'todo','live refund redirect url');

    $RedirectURL = $MockedPaymentMethodSIA->getCancelRedirectURL();
    is($RedirectURL,'todo','live cancel redirect url');

    return;
}

sub test_getCurrentRegistrationTransaction_noRegistrationTransactions {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    $MockedPaymentMethodSIA->set({'CurrentRegistrationTransaction' => undef });

    my $Registrationtransaction = $MockedPaymentMethodSIA->getCurrentRegistrationTransaction();

    is($Registrationtransaction,undef,'undef current transaction');

    return;
}

sub test_getCurrentRegistrationTransaction_transactionMissing {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    $MockedPaymentMethodSIA->set( { 'CurrentRegistrationTransaction' => 'InvalidAlias' } );

    my $Registrationtransaction = $MockedPaymentMethodSIA->getCurrentRegistrationTransaction();

    is($Registrationtransaction,undef,'no child current transaction');

    return;
}

sub test_getCurrentRegistrationTransaction_success {
    my $self = shift;

    my $MockedPaymentMethodSIA = CreateMockedPaymentMethod();

    ok(defined $MockedPaymentMethodSIA->getCurrentRegistrationTransaction(),'Transaction defined');

    return;
}


__PACKAGE__->runScript();

1;
