#==================================================================================================
# package      DE_EPAGES::SIA::t::unit::Sale
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::SIA::API::Request::Sale
#==================================================================================================
package DE_EPAGES::SIA::t::unit::Sale;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools         qw (isValid);
use DE_EPAGES::SIA::API::Object::LineItemPaymentSIA;
use DE_EPAGES::SIA::t::unit::MockupObjects  qw (CreateMockedLineItemPayment
                                                TEST_MERCHANT_FUC
                                                CreateMockedServlet
                                                TEST_TERMINAL_NUMBER
                                                TEST_TRANS_ID
                                                TEST_REMOTE_ADDRESS);
use DE_EPAGES::SIA::API::Constants          qw (WINDOW_NAME
                                                SALE_EVENT_NAME
                                                PROTOCOL_VERSION
                                                EVENT_NAME_LENGTH
                                                WINDOW_LENGTH
                                                PROTOCOL_VERSION_LENGTH
                                                ID_TERMINAL_LENGTH
                                                CLIENT_ENTITY_ID_LENGTH
                                                MERCHANT_FUC_LENGTH
                                                USER_ID_LENGTH
                                                TRANSACTION_TYPE_LENGTH
                                                OPC_TOKEN_ID_LENGTH
                                                WEB_REQUEST_ID_LENGTH
                                                CARD_EXPIRE_DATE_LENGTH
                                                LANGUAGE_LENGTH);
use Test::More;
use feature                                 qw (switch);
use strict;

## no critic(ProhibitMagicNumbers)
sub test {
    my $self = shift;

    my $MockedLineItemPaymentSIA = CreateMockedLineItemPayment();
    my $Servlet                  = CreateMockedServlet();
    $Servlet->vars('IsMobile', 1);

    $self->test_getPaymentSalesRequest($MockedLineItemPaymentSIA);
    $self->test_getPaymentSalesRequest($MockedLineItemPaymentSIA, $Servlet);

    done_testing();
}

sub test_getPaymentSalesRequest {
    my $self = shift;
    my ($MockedLineItemPaymentSIA, $Servlet) = @_;

    my $aRequestData = $MockedLineItemPaymentSIA->getPaymentSalesRequest()->getRequestData($Servlet);

    ## no critic qw(ProhibitLongLines)
    foreach my $hRequestSaleData (@{$aRequestData}) {
        given ($hRequestSaleData->{'Field'}) {
            when ($_ eq 'sNombreEvento') {
                $self->testField($_,$hRequestSaleData, SALE_EVENT_NAME,EVENT_NAME_LENGTH);
            }
            when ($_ eq 'sIdWindow') {
                $self->testField($_,$hRequestSaleData, WINDOW_NAME,WINDOW_LENGTH);
            }
            when ($_ eq 'IDVERSION') {
                $self->testField($_,$hRequestSaleData, PROTOCOL_VERSION,PROTOCOL_VERSION_LENGTH);
            }
            when ($_ eq 'IDTERMINAL') {
                $self->testField($_,$hRequestSaleData, TEST_TERMINAL_NUMBER,ID_TERMINAL_LENGTH);
            }
            when ($_ eq 'IDENTIDAD') {
                $self->testField($_,$hRequestSaleData, 'ACEN',CLIENT_ENTITY_ID_LENGTH);
            }
            when ($_ eq 'IDCOMERCIO') {
                $self->testField($_,$hRequestSaleData, TEST_MERCHANT_FUC,MERCHANT_FUC_LENGTH);
            }
            when ($_ eq 'IDSERVICIO') {
                $self->testField($_,$hRequestSaleData,  '');
            }
            when ($_ eq 'IDUSUARIO') {
                $self->testField($_,$hRequestSaleData,'',USER_ID_LENGTH);
            }
            when ($_ eq 'IDTM') {
                $self->testField($_,$hRequestSaleData,'1200',TRANSACTION_TYPE_LENGTH);
            }
            when ($_ eq 'OPCTOKEN') {
                $self->testField($_,$hRequestSaleData,'01',OPC_TOKEN_ID_LENGTH);
            }
            when ($_ eq 'IDPEDIDO') {
                $self->testField($_,$hRequestSaleData,TEST_TRANS_ID,WEB_REQUEST_ID_LENGTH);
            }
            when ($_ eq 'NOMTITULAR') {
                $self->testField($_,$hRequestSaleData,'Test User');
            }
            when ($_ eq 'CODPAIS') {
                $self->testField($_,$hRequestSaleData,'276');
            }
            when ($_ eq 'PAN') {
                $self->testField($_,$hRequestSaleData,'');
            }
            when ($_ eq 'FECCAD') {
                $self->testField($_,$hRequestSaleData,'',CARD_EXPIRE_DATE_LENGTH);
            }
            when ($_ eq 'CVC') {
                $self->testField($_,$hRequestSaleData,'');
            }
            when ($_ eq 'IMPTRANS') {
                $self->testField($_,$hRequestSaleData,'8000');
            }
            when ($_ eq 'CODMON') {
                $self->testField($_,$hRequestSaleData,'978');
            }
            when ($_ eq 'TIMETRANS') {
                $self->testField($_,$hRequestSaleData,'2015-11-26T07:24:31.000 Europe/Berlin');
            }
            when ($_ eq 'IDIOMA') {
                $self->testField($_,$hRequestSaleData,'de',LANGUAGE_LENGTH);
            }
            when ($_ eq 'IPORIGEN') {
                $self->testField($_,$hRequestSaleData,TEST_REMOTE_ADDRESS);
            }
            when ($_ eq 'FIRMTOKEN') {
                # TODO: Activate test for firm token.
                #$self->testEncryptionToken($hRequestSaleData, 'todo');
            }
            when ($_ eq 'TOKEN') {
                $self->testField($_,$hRequestSaleData,'');
            }
            when ($_ eq 'DESTOKEN') {
                $self->testField($_,$hRequestSaleData,'');
            }
            when ($_ eq 'URLOK') {
                if (defined($Servlet) && $Servlet->vars('IsMobile')) {
                    $self->testField(
                        $_,
                        $hRequestSaleData,
                        'http://unit-test.epages.de/epages/UnitTestShop.mobile/'.
                        '?ChangeAction=PaymentBacklinkSuccessSIA&ObjectID=12345&OrderID=15917&ViewAction=MobileSF-View');
                }
                else {
                    $self->testField(
                        $_,
                        $hRequestSaleData,
                        'http://unit-test.epages.de/epages/UnitTestShop.sf/'.
                        '?ChangeAction=PaymentBacklinkSuccessSIA&ObjectID=12345&OrderID=15917');
                }
            }
            when ($_ eq 'URLKO') {
                if (defined($Servlet) && $Servlet->vars('IsMobile')) {
                    $self->testField(
                        $_,
                        $hRequestSaleData,
                        'http://unit-test.epages.de/epages/UnitTestShop.mobile/'.
                        '?ChangeAction=PaymentBacklinkErrorSIA&ObjectID=12345&OrderID=15917&ViewAction=MobileSF-View');
                }
                else {
                    $self->testField(
                        $_,
                        $hRequestSaleData,
                        'http://unit-test.epages.de/epages/UnitTestShop.sf/'.
                        '?ChangeAction=PaymentBacklinkErrorSIA&ObjectID=12345&OrderID=15917');
                }
            }
            when ($_ eq 'EMAILUSUARIO') {
                $self->testField($_,$hRequestSaleData,'test@epages.de');
            }
            default {
                ok(0,'unknown field: '.$_);
            }
        }
    }
    return;
}

sub testField {
    my $self = shift;
    my ($FieldName,$hData,$ExpectedValue,$Length) = @_;

    if (isValid $Length) {
        my $Blanks = ' ' x ($Length - length($ExpectedValue)) ;
        is($hData->{'Field'},$FieldName                 ,'EventName - '.$FieldName.'Field');
        is($hData->{'Value'},$ExpectedValue.$Blanks     ,'EventName - '.$FieldName.'Value');
    }
    else {
        is($hData->{'Field'},$FieldName         ,'EventName - '.$FieldName.'Field');
        is($hData->{'Value'},$ExpectedValue     ,'EventName - '.$FieldName.'Value');
    }

    return;
}

__PACKAGE__->runScript();

1;
