#========================================================================================
# package      DE_EPAGES::SIA::t::unit::Tools
# state        public
# description  Unit tests for DE_EPAGES::SIA::API::Tools.
#========================================================================================
package DE_EPAGES::SIA::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::SIA::API::Tools qw (GetToolsInstance);
use Test::More;

use strict;

## no critic(ProhibitMagicNumbers)
sub test {
    my $self = shift;

    $self->testString('Hello World');
    $self->testHexString('48656c6c6f20576f726c64');

    $self->test_stringToHexString_success();
    $self->test_hexStringToString_success();
    $self->test_conversionRoundtrip();
    $self->test_formatAmount();
    $self->test_fillFieldWithBlanks();
    $self->test_fillFieldWithLeadingZeros();
    $self->test_handleTooLongFields();
    $self->test_getCurrentDate();
    done_testing();

    return;
}

sub test_handleTooLongFields {
    my $self = shift;

    is (GetToolsInstance()->handleTooLongFields('TestString',12),'TestString', 'field was not changed');
    is (GetToolsInstance()->handleTooLongFields('TestStringTestString',12),'TestStringTe', 'truncat field');
    is (GetToolsInstance()->handleTooLongFields('',12),'', 'empty string not changed');

    is (GetToolsInstance()->handleTooLongFields(0,2),0, 'digits not changed');
    is (GetToolsInstance()->handleTooLongFields(123,2),12, 'truncat integer field');

    return;
}

sub test_fillFieldWithBlanks {
    my $self = shift;

    is (GetToolsInstance()->fillFieldWithBlanks('TestString',12),'TestString  '     , 'fill Field with additional blanks');
    is (GetToolsInstance()->fillFieldWithBlanks('TestString',15),'TestString     '  , 'fill Field with additional blanks - variation');
    is (GetToolsInstance()->fillFieldWithBlanks('TestString12',12),'TestString12'   , 'fill Field with no blanks ');
    return;
}

sub test_fillFieldWithLeadingZeros {
    my $self = shift;

    is (GetToolsInstance()->fillFieldWithLeadingZeros(12,4),    '0012' , 'fill int Field with leading zeros');
    is (GetToolsInstance()->fillFieldWithLeadingZeros('12',4),  '0012' , 'fill string Field with leading zeros');
    is (GetToolsInstance()->fillFieldWithLeadingZeros('1212',4),'1212' , 'fill Field with no leading zeros');
    return;
}

sub test_formatAmount {
    my $self = shift;

    is (GetToolsInstance()->formatAmount('80'),     '8000' , 'fromat amount without decimal numbers');
    is (GetToolsInstance()->formatAmount('80.00'),  '8000' , 'fromat amount with decimal numbers');
    is (GetToolsInstance()->formatAmount('80.05'),  '8005' , 'fromat amount variation decimal numbers');
    is (GetToolsInstance()->formatAmount('80.3'),   '8030' , 'fromat amount one decimal number');
    is (GetToolsInstance()->formatAmount('80.002'), '8000' , 'fromat amount three decimal numberss');
    is (GetToolsInstance()->formatAmount('80.392'), '8039' , 'fromat amount variation three decimal numberss');

    return;
}

sub test_stringToHexString_success {
    my $self = shift;

    my $String    = $self->testString;
    my $HexString = GetToolsInstance()->stringToHexString($String);

    is( $String,    $self->testString,    'stringToHexString input unchanged' );
    is( $HexString, $self->testHexString, 'stringToHexString result' );

    return;
}

sub test_hexStringToString_success {
    my $self = shift;

    my $HexString = $self->testHexString;
    my $String    = GetToolsInstance()->hexStringToString($HexString);

    is( $HexString, $self->testHexString, 'hexStringToString input unchanged' );
    is( $String,    $self->testString,    'hexStringToString result' );

    return;
}

sub test_conversionRoundtrip {
    my $self = shift;

    my $String            = $self->testString;
    my $HexString         = GetToolsInstance()->stringToHexString($String);
    my $ReconvertedString = GetToolsInstance()->hexStringToString($HexString);

    is( $String, $ReconvertedString, 'conversion roundtrip' );
}

sub test_getCurrentDate {
    my $self = shift;

    my $Result = GetToolsInstance()->getCurrentDate();

    ok( $Result =~ /^\d{8}$/, 'getCurrentDate' );

    return;
}

sub testString {
    my $self = shift;
    my ($String) = @_;

    $self->{'testString'} = $String if isValid($String);

    return $self->{'testString'};
}

sub testHexString {
    my $self = shift;
    my ($HexString) = @_;

    $self->{'testHexString'} = $HexString if isValid($HexString);

    return $self->{'testHexString'};
}

__PACKAGE__->runScript();
