#========================================================================================
# package      DE_EPAGES::Shop::t::unit::ShopTaxClass
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Shop::API::ShopTaxClass
#========================================================================================
package DE_EPAGES::Shop::t::unit::ShopTaxClass;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_loadTaxClassbyGUID();
    $self->test_loadTaxClassbyGUID_unknownGUID();

    $self->test_getAvailableTaxClasses();
    $self->test_getAvailableTaxClasses_US();
    $self->test_getAvailableTaxClasses_External();

    done_testing();
}



sub test_loadTaxClassbyGUID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseExternalTaxCalculation = 0;
    my $IsTaxMatrixUS = 0;
    my $Mocks = $self->_createMocks($IsTaxMatrixUS, $UseExternalTaxCalculation);

    my $ShopTaxClass = $Mocks->get('SUT');
    my $TaxClass = $ShopTaxClass->loadTaxClassbyGUID('31415');

    is($TaxClass->guid(), '31415', "$SubTestName - test if the right tax class was returned");
}



sub test_loadTaxClassbyGUID_unknownGUID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseExternalTaxCalculation = 0;
    my $IsTaxMatrixUS = 0;
    my $Mocks = $self->_createMocks($IsTaxMatrixUS, $UseExternalTaxCalculation);

    my $ShopTaxClass = $Mocks->get('SUT');
    my $TaxClass = $ShopTaxClass->loadTaxClassbyGUID('10010');

    is($TaxClass, undef, "$SubTestName - test if no tax class was returned");
}



sub test_getAvailableTaxClasses {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseExternalTaxCalculation = 0;
    my $IsTaxMatrixUS = 0;
    my $Mocks = $self->_createMocks($IsTaxMatrixUS, $UseExternalTaxCalculation);

    my $ShopTaxClass = $Mocks->get('SUT');
    my $aTaxClasses = $ShopTaxClass->getAvailableTaxClasses();

    my $aTaxClassNames = [ map { $_->alias()} @$aTaxClasses ];

    my $aExpectedTaxClassNames = [
        'TaxClass1',
        'TaxClass2',
        'TaxClassDigital'
    ];

    is($Mocks->get('Shop')->get('TaxMethod'), undef, "$SubTestName - test if the shop has no tax method");
    is_deeply($aTaxClassNames, $aExpectedTaxClassNames, "$SubTestName - test if all tax classes were returned");
}



sub test_getAvailableTaxClasses_US {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseExternalTaxCalculation = 0;
    my $IsTaxMatrixUS = 1;
    my $Mocks = $self->_createMocks($IsTaxMatrixUS, $UseExternalTaxCalculation);

    my $ShopTaxClass = $Mocks->get('SUT');
    my $aTaxClasses = $ShopTaxClass->getAvailableTaxClasses();

    my $aTaxClassNames = [ map { $_->alias()} @$aTaxClasses ];

    my $aExpectedTaxClassNames = [
        'TaxClass1',
        'TaxClass2',
    ];

    is($Mocks->get('Shop')->get('TaxMethod'), undef, "$SubTestName - test if the shop has no tax method");
    is_deeply($aTaxClassNames, $aExpectedTaxClassNames, "$SubTestName - test if all tax classes except digital were returned");
}



sub test_getAvailableTaxClasses_External {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseExternalTaxCalculation = 1;
    my $IsTaxMatrixUS = 0;
    my $Mocks = $self->_createMocks($IsTaxMatrixUS, $UseExternalTaxCalculation);

    my $ShopTaxClass = $Mocks->get('SUT');
    my $aTaxClasses = $ShopTaxClass->getAvailableTaxClasses();

    my $aTaxClassNames = [ map { $_->alias()} @$aTaxClasses ];

    my $aExpectedTaxClassNames = [
        'TaxClass1',
        'TaxClass2',
    ];

    is($Mocks->get('Shop')->get('TaxMethod')->alias(), 'MyTaxMethod', "$SubTestName - test if the shop has a tax method");
    is_deeply($aTaxClassNames, $aExpectedTaxClassNames, "$SubTestName - test if all tax classes were returned");
}




sub _createMocks {
    my $self = shift;
    my ($IsTaxMatrixUS, $UseExternalTaxCalculation) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::ShopTaxClass', [$Shop]);
        $Mockify->addMethodSpy('_getAvailableTaxClassesOfTaxCalculationByEpages');
        $Mockify->addMethodSpy('_getAvailableTaxClassesOfTaxCalculationByExternalService');
        $Mockify->addMockWithReturnValueAndParameterCheck('_ExistsClassByAlias', 1, [{'string' => 'TaxMatrixUS'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', ['TaxMethod', 'TaxMatrix', 'TaxMatrixDigital'], sub {
        my ($TaxMethod, $TaxMatrix, $TaxMatrixDigital) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
            'TaxMethod' => $UseExternalTaxCalculation ? $TaxMethod : undef,
            'TaxMatrix' => $TaxMatrix,
            'TaxMatrixDigital' => $TaxMatrixDigital,
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxMethod', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMethod', {
            'Alias' => 'MyTaxMethod',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxMatrix', ['TaxClass1', 'TaxClass2'], sub {
        my ($TaxClass1, $TaxClass2) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMatrix', {
            'TaxClasses' => [$TaxClass1, $TaxClass2],
        });
        $Mockify->addMockWithReturnValueAndParameterCheck('instanceOf', $IsTaxMatrixUS, [{'string' => 'TaxMatrixUS'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxMatrixDigital', ['TaxClassDigital'], sub {
        my ($TaxClassDigital) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxMatrixDigital', {
            'TaxClassesDigital' => [$TaxClassDigital],
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClass1', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'Alias' => 'TaxClass1',
            'GUID' => '12345',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClass2', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'Alias' => 'TaxClass2',
            'GUID' => '31415',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClassDigital', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClassDigital', {
            'Alias' => 'TaxClassDigital',
            'GUID' => '24816',
        });
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

__PACKAGE__->runScript();