use Test::More tests => 5;
use strict;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Database::API::Connection qw( ConnectToSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

local $ENV{'EPAGES_TESTSITE'}='Site';

sub Main {
    ConnectToSite();

    ok (! Cleanup(), 'cleaned');

    my $System   = LoadRootObject();
    my $Store    = $System->folder('Stores')->child('Store');
    my $Provider = $System->folder('Providers')->child('Distributor');
    my $ShopType = $Provider->folder('ShopTypes')->child('Demo');

    my $ShopRef = $Provider->insertShop({
        'Alias'     => 'Test',
        'ShopAlias' => 'Test',
        'Store'     => $Store,
        'ShopType'  => $ShopType,
    });

    ok (defined $ShopRef, 'shop created');

    eval{
        $Provider->insertShop({
            'Alias'     => 'test',
            'ShopAlias' => 'test',
            'Store'     => $Store,
            'ShopType'  => $ShopType,
        });
    };

    ok (ExistsError(), 'No second shop created');

    is (GetError()->code, 'ShopRefAlias_Exists', 'Not possible to create two shops just with different upper and lower case.');

    ok (Cleanup(), 'cleaned');
    DisconnectSite();
}

RunScript(
    Sub => \&Main
);

sub Cleanup {
    my $System   = LoadRootObject();
    my $Provider = $System->folder('Providers')->child('Distributor');
    if ($Provider->folder('ShopRefs')->existsChild('test')) {
        $Provider->folder('ShopRefs')->child('test')->deleteRemoteShop;
        $Provider->folder('ShopRefs')->child('test')->delete;
    }
    if ($Provider->folder('ShopRefs')->existsChild('Test')) {
        $Provider->folder('ShopRefs')->child('Test')->deleteRemoteShop;
        $Provider->folder('ShopRefs')->child('Test')->delete;
        return 1;
    } else {
        return 0;
    }
}
