﻿use Test::More;

use strict;
use utf8;
use SOAP::Lite;

use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::WebService::API::WebService qw(FormatSimpleArray FormatString);

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            _TestNormalString();
            _TestUTF8String();
            _TestEmptyArray();
            _TestNormalArray();
            _TestUTF8Array();
            _TestComplexArray();

            done_testing();
        }
    );
}

sub _TestEmptyArray {
    my $aValues = FormatSimpleArray([]);

    is(scalar(@{$aValues}),0,'empty array');
}

sub _TestComplexArray {
    my $aValues = FormatSimpleArray([{'test' => 'test123'},'blub2']);

    is(ref($aValues->[0]),'HASH','complex simple array');
    is($aValues->[1],'blub2','complex simple array');
}
sub _TestNormalArray {
    my $aValues = FormatSimpleArray(['blub','blub2']);

    is($aValues->[0],'blub','normal simple array');
    is($aValues->[1],'blub2','normal simple array');
}

sub _TestUTF8Array {
    my $aValues = FormatSimpleArray(['blüb','blüb2']);

    is($aValues->[0]->isa('SOAP::Data'),1,'utf8 simple array');
    is($aValues->[1]->isa('SOAP::Data'),1,'utf8 simple array');
    is($aValues->[0]->value(),'blüb','utf8 simple array');
    is($aValues->[1]->value(),'blüb2','utf8 simple array');
}

sub _TestNormalString {
    my $Value = FormatString('Test123');

    is($Value,'Test123','normal string');
}

sub _TestUTF8String {
    my $Value = FormatString('blüb');
    is($Value->isa('SOAP::Data'),1,'utf8 string');
    is($Value->value(),'blüb','utf8 string');
}


RunScript( Sub => \&Main );

1;
